/* ---- LCD Driver Routine for 2x16 / 4x20 char. Display ----------------------------------------------------- */
/*                                                                                                             */
/* (c) B. Nollmeyer, WINAVR sample Routine. Das Display wird im 4 BIT Modus betrieben.                         */
/*                                                                                                             */

#ifndef lcd_h
#define lcd_h

/* ---- Define BIT Variablen --------------------------------------------------------------------------------- */
/*                                                                                                             */
#define HIGH 1                                                  /* digital High Pegel                          */
#define LOW  0                                                  /* digital Low Pegel                           */

/* ---- Define I/O's, AVR I/O's sind nicht BITadressierbar! -------------------------------------------------- */
/*                                                                                                             */
struct bits {
  uint8_t b0:1;
  uint8_t b1:1;
  uint8_t b2:1;
  uint8_t b3:1;
  uint8_t b4:1;
  uint8_t b5:1;
  uint8_t b6:1;
  uint8_t b7:1;
} __attribute__((__packed__));

#define SBIT(port,pin) ((*(volatile struct bits*)&port).b##pin) /* I/O Defines vereinfachen                    */

/* ---- Define Display Signale ------------------------------------------------------------------------------- */
/*                                                                                                             */
#define ENABLE SBIT (PORTB, 3)                                  // Display Enable
#define RS SBIT (PORTD, 6)                                      // Display RS BIT

/* ---- Define Display Kommandos ----------------------------------------------------------------------------- */
/*                                                                                                             */
#define function_set_1      0x28                                // 4 Bit Modus / 2 Zeilen / 5x7 Dots
#define function_set_2      0x2C                                // 4 Bit Modus / 2 Zeilen / 5x10 Dots
#define function_set_3      0x20                                // 4 Bit Modus / 1 Zeile / 5x7 Dots
#define function_set_4      0x24                                // 4 Bit Modus / 1 Zeile / 5x10 Dots

#define on_off_control_1    0x0C                                // Display An / Cursor Aus / Blink Aus
#define on_off_control_2    0x0E                                // Display An / Cursor An / Blink Aus
#define on_off_control_3    0x0F                                // Display An / Cursor An / Blink An
#define on_off_control_4    0x08                                // Display Aus

#define entrymode_1         0x06                                // Cursor nach rechts / kein Shift
#define entrymode_2         0x07                                // Cursor nach rechts / Shift
#define entrymode_3         0x04                                // Cursor nach links / kein Shift
#define entrymode_4         0x05                                // Cursor nach links / Shift

#define cursor_line_1       0x80                                // LCD Zeile 1 / Referenz LCD Datenblatt
#define cursor_line_2       0xC0                                // LCD Zeile 2 / Referenz LCD Datenblatt
#define cursor_line_3       0x90                                // LCD Zeile 3 / Referenz LCD Datenblatt
#define cursor_line_4       0xD0                                // LCD Zeile 4 / Referenz LCD Datenblatt

#define clear_display       0x01                                // Clear Display
#define cursor_home         0x02                                // Cursor Home

/* ---- Define Prototypes ------------------------------------------------------------------------------------ */
/*                                                                                                             */
extern void lcd_command (uint8_t dsp_control_out);              // Funktion fr Display Steuersignale
extern void lcd_char (uint8_t send_char);                       // Funktion Zeichen ausgeben
extern void lcd_init (void);                                    // Funktion LCD Initialisieren
extern void lcd_puts (const char *lcd_string);                  // Funktion Display String ausgeben

#endif
