/* ---- LCD Driver Routine for 2x16 / 4x20 char. Display ----------------------------------------------------- */
/*                                                                                                             */
/* (c) B. Nollmeyer, WINAVR sample Routine! Das Display wird im 4 BIT Modus betrieben.                         */
/* ATMEL ATTINY 2313, die Routine bentigt 247 Byte inkl. Startup Code. WINAVR gcc 3.4.6                       */

/* ---- CPU CLOCK -------------------------------------------------------------------------------------------- */
/*                                                                                                             */
#ifndef F_CPU
#define F_CPU 1000000UL                                         /* Quarz Frequenz = 1 MHz                      */
#endif

/* ---- Include Files ---------------------------------------------------------------------------------------- */
/*                                                                                                             */
#include <stdint.h>                                             /* ISO C99 Integer types                       */
#include <avr/io.h>                                             /* ATTINY I/O's                                */
#include <util/delay.h>                                         /* delay Lib.                                  */
#include "lcd.h"                                                /* LCD Lib.                                    */

/* ---- MAIN ------------------------------------------------------------------------------------------------- */

int main (void)  {
  DDRB = (1 << DDB3)                                            // Anschlu Display ENABLE 
       | (1 << DDB4)                                            // Anschlu Display Datenbit DB4 
       | (1 << DDB5)                                            // Anschlu Display Datenbit DB5
       | (1 << DDB6)                                            // Anschlu Display Datenbit DB6
       | (1 << DDB7);                                           // Anschlu Display Datenbit DB7
  DDRD = (1 << DDD6);                                           // Anschlu Display RS
  
  _delay_ms (15);                                               // wait 15 mSec. after Power ON
  lcd_init ();                                                  // Display initialisieren
  lcd_puts ("WIN_AVR GCC");                                     // Display Text

  for (;;);
}

/* ---- Display Ausgabe String ------------------------------------------------------------------------------- */

void lcd_puts (const char *lcd_string)  {
  while (*lcd_string)  {
    lcd_char (*lcd_string++);                                   // String auslesen und an LCD Ausgabe bergeben
  }
}

/* ---- Display Ausgabe Steuersignale ------------------------------------------------------------------------ */

void lcd_command (uint8_t dsp_control_out)  {    
  RS = LOW;                                                     // Steuersignale an Display ausgeben (RS BIT=0)
  lcd_char (dsp_control_out);                                   // Steuersignal ausgeben
  _delay_ms (4.1);                                              // Warteschleife 4,1 mSec.
  RS = HIGH;                                                    // RS BIT=1 => Daten
}

/* ---- Display Ausgabe Zeichen ------------------------------------------------------------------------------ */

void lcd_char (uint8_t send_char)  {
  uint8_t n=2;                                                  // Sende 2x 4 BIT (oberes Nibble zuerst)
  do  {
        PORTB = (PORTB & 0x0F) | (send_char & 0xF0);            // Port low nibble nicht verndern
        ENABLE = HIGH;                                          // ENABLE = HIGH 
        ENABLE = LOW;                                           // ENABLE = LOW, Steuersignal Datenbernahme
        send_char <<= 4;                                        // unteres Nibble nach oben schieben
  }while (--n);
  _delay_us (42);                                               // Warteschleife 42 uSec.
}

/* ---- Display initialisieren ------------------------------------------------------------------------------- */

void lcd_init (void)  {
  uint8_t n = 3;                                                // Init. Sequenz, Referenz LCD Datenblatt
  do  {                                                         // 3x 33H (0x03), init cycle 3x
        lcd_command (0x33);                                     // DB4, DB5 = 1
  }while (--n);
  lcd_command (function_set_1);                                 // 4 Bit Modus / 2 Zeilen / 5x7 Dots
  lcd_command (on_off_control_1);                               // Display An / Cursor Aus / Blink Aus
  lcd_command (clear_display);                                  // Clear Display
  lcd_command (entrymode_1);                                    // Cursor nach rechts / kein Shift
}
